%This file replicates Figure 2 in Shi (2014)
%
%The figure illustrates the big tail problem caused by the random
%denominator. The distribution to be simulated is 
%
%          J_Lambda/\sqrt(J_omega) = (2sigmaZ0-Z0^2)/(2|sigma-Z0|)
%
%Calculation shows that the cdf of the above distribution is:
%
%         F(x)=normcdf(sigma+x-sqrt(sigma^2+x^2)) +1 -
%                  normcdf(sigma-x+sqrt(sigma^2+x^2))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Draw the density of JJ for a fixed sigma
clear
sigma=1.5;   

Zsim = randn(1000000,1);
Tsim = (2*sigma*Zsim - Zsim.^2)./(2*abs(sigma-Zsim));
nomega_sim = (sigma-Zsim).^2;
omega_critv = quantile(Zsim.^2,0.95);


xx=-5:0.1:5;

a=sqrt(xx.^2+sigma^2);
b=xx-a;
c = 1-xx./a;

pdfJJ=normpdf(sigma+b)+normpdf(sigma-b);

pdfJJ=c.*pdfJJ;

pdfnorm=normpdf(xx);

Fx=@(x)normcdf(sigma+x-sqrt(sigma^2+x^2))+1-normcdf(sigma-x+sqrt(sigma^2+x^2));
ncrej = [1-Fx(norminv(0.975))+Fx(norminv(0.025)),mean(nomega_sim>omega_critv)...
                  mean((nomega_sim>omega_critv).*(abs(Tsim)>norminv(0.975)))];

%%
hfig = figure;
set(hfig,'units','centimeters');
pos = get(hfig,'position');
set(hfig,'position',[pos(1:2),9,6]);
axis([-5 5 0 0.5])
h0=plot(xx,pdfJJ,'-',xx,pdfnorm,'--');
set(gca, 'Box', 'on','xtick',-5:1:5);
set(h0,'linewidth',2)
ylabel('density','fontsize',11,'fontweight','b','interpreter','none');
hleg=legend('$J_\Lambda/J_\omega^{1/2}$','$N(0,1)$');
set(hleg,'location','northwest');
set(hleg,'interpreter','none')
set(hleg,'box','off')
matlabfrag('figure2.tex');
