%This is the main program for the simulation Example 2 of Shi (2014).
%Set n = 100 for Figure 3(a), 250 for Figure 3(c), 500 for Figure 3(e)

%In this program, we consider the following two models:
%
% F: (Y_1,Y_2)\sim N((theta_1,0),I_2), theta_1\in R
% G: (Y_1,Y_2)\sim N((0,theta_2),I_2), theta_2\in R
%
%The DGP is:
%
% (Y_1,Y_2) \sim N(theta_10,theta_20),Diag((25,1)));
%
%Calculation shows that LR = (theta_10^2-theta_20^2), and omega^2 =
%25theta_10^2+theta_20^2.
%
%    When theta10=theta_20, the H_0 holds. When theta_10>theta_20, model F is better.
%
%In this program, I compute the coverage probability of the nondegenerate
%test proposed in Shi (2013), the one-step Vuong test, the two-step test
%and the split-sample test.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%addpath 'C:\Users\xshi.PRIMO\Documents\My Dropbox\tools\matlabmfiles';
clear
n = 250;    %sample size
S1 = 5000;  %Monte Carlo repetition
S2 = 5001;  %Critical value simulation repetition

alpha = 0.05; %significance level

VarY = [25,0;0,1];
tic
th20_vec = 0:0.15:3;
j=1;
rejprob = NaN(length(th20_vec),13);
   %th10~th20~osVg1~osVg2~tsVg1~tsVg2~varT~ndVg1~ndVg2~cstar~split1~split2~n
for th20 = th20_vec;
    th10 = th20;
    
    [sample_rstr,cv_rstr1,cv_rstr2]=RandStream.create('mrg32k3a','NumStreams',3);
    
    T_Vg_vec = NaN(S1,1);    %Test statistic vector for classical Vuong
    VarT_vec = NaN(S1,1);    %Variance test stat in two-step classical Vuong
    T_ndVg_vec = NaN(S1,1);  %Test stat for nondegenerate test
    T_split_vec = NaN(S1,1); %Test stat for split sample test
    cv_VarT = NaN(S1,1);     %critical value for varianc etest in two-step Vg
    cv_ndVg_vec = NaN(S1,1); %critical value for nondegenerate test
    cstar_vec = NaN(S1,1);   %chosen c in nondegenerate test
    for s = 1:S1;
        %Generate the data
        Y0 = randn(sample_rstr,n,2);
        Y = Y0*sqrtm(VarY)+repmat([th10,th20],n,1);
        
        %Parameter Estimators: (In general, should numerically maximize
        %log-likelihood function. But here the problem is simple enough to have
        %closed form solution. I use closed form solution here.)
        
        theta_hat = mean(Y(:,1));
        beta_hat = mean(Y(:,2));
        
        %Test preparation
        [logf,d_logf,d2_logf] = NorLoc_logfi(Y,theta_hat);
        [logg,d_logg,d2_logg] = NorLoc_loggi(Y,beta_hat);
        
        Zcv = randn(cv_rstr1,S2,2);
        %normal vector used to simulate cv for variance test
        
        nLR_hat = sum(logf-logg);      %sample log-likelihood ratio times n
        
        nomega2_hat = (logf-logg)'*(logf-logg) - nLR_hat^2/n;
        %n times sample variance of \Lambda_i
        A_hat = [sum(d2_logf),0;0,-sum(d2_logg)]/n;
        
        meandL = mean([d_logf,-d_logg]);
        dmeandL = [d_logf,-d_logg] - repmat(meandL,n,1);
        B_hat = dmeandL'*dmeandL/n;
        
        W_hat = (sqrtm(B_hat)/A_hat)*sqrtm(B_hat);
        [~,V_hat]=eig(W_hat);
                
        %Classical Vuong tests
        T_Vg_vec(s) = nLR_hat/sqrt(nomega2_hat); %test stat for classical Vuong
        %(one step and two-step)
        VarT_vec(s) = nomega2_hat;
        cv_VarT(s) = quantile((Zcv.^2)*(diag(V_hat).^2),1-alpha);
               
        %Non-degenerate Vuong Tests
        T_ndVg = @(c)(nLR_hat+trace(V_hat)/2)/...
                                   sqrt(nomega2_hat + c*trace(V_hat^2));
        %test stat for the nondegenerate Vuong test
        
        [cv_ndVg_vec(s),cstar_vec(s)]=ndVuong_cv(alpha,V_hat,cv_rstr2,S2);
        
        T_ndVg_vec(s) = T_ndVg(cstar_vec(s));
        
        
        %Split-sample test    (Split the sample into two halves based on the
        %                              natural ordering of the data.)
        sigl1_sq=var(logf);
        sigl2_sq=var(logg);
        covll = corr(logf,logg)*sqrt(sigl1_sq*sigl2_sq);
        
        epsi_n = (max(4*covll/(sigl1_sq+sigl2_sq),0)-1); %robust weight
        
        llnsplit = (1/(2+epsi_n))*(logf(1:n/2,1)-logg((1+n/2):n,1)...
            +(1+epsi_n)*(logf((1+n/2):n,1)-logg(1:n/2,1)));
        LR_split = mean(llnsplit);      %split sample log likelihood ratio
        omegasq_split = var(llnsplit);  %variance of the split sample log density ratio
        T_split = sqrt(n/2)*LR_split/sqrt(omegasq_split);   %split sample test statistic
        T_split_vec(s)=T_split;
        if 20*floor(s/20) == s;
            display([floor([toc,s]),th20]);
        end
    end
    
    %Computing the rejection probabilities:
    z_normal = norminv(1-alpha/2);
    rejprob_osVg = [mean(T_Vg_vec>z_normal),mean(T_Vg_vec<-z_normal)];
    rejprob_varT = mean(VarT_vec>cv_VarT);
    rejprob_tsVg = [mean((T_Vg_vec>z_normal).*(VarT_vec>cv_VarT)),...
        mean((T_Vg_vec<-z_normal).*(VarT_vec>cv_VarT))];
    rejprob_ndVg = [mean(T_ndVg_vec>cv_ndVg_vec),mean(T_ndVg_vec<-cv_ndVg_vec)];
    rejprob_split = [mean(T_split_vec>z_normal),mean(T_split_vec<-z_normal)];
    mean_cv_ndVg = mean(cv_ndVg_vec);
    mean_cstar = mean(cstar_vec);
    
    rejprob(j,:)=[th10,th20,rejprob_osVg,rejprob_tsVg,rejprob_varT,...
                                    rejprob_ndVg,mean_cstar,rejprob_split,n];
    j=j+1;
end;
totaltime = toc;
%Reporting the results:
%% reporting the results

%saving the results in a txt file
if th10==th20;
    eval(['fID=fopen(''NorLoc_neqs_',num2str(n),'_nulldata.txt'',''w'');']);
else
    eval(['fID=fopen(''NorLoc_neqs_',num2str(n),'_powerdata.txt'',''w'');']);
end
fprintf(fID,'th10|th20|osVg1|osVg2|tsVg1|tsVg2|varT|ndVg1|ndVg2|cstar|split1|split2|n\n');
for j=1:length(th20_vec);
    fprintf(fID,...
 '%1.2f|%1.2f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%1.3f|%g\n',...
                                                           rejprob(j,:));                                                  
end;
fprintf(fID,'_______________________________________________________________\n');
fprintf(fID,'Total Time Elapsed in Minutes = %g',totaltime/60);
fclose(fID);

data = rejprob;
%% Graphing the results
n = data(1,13);
th10 = data(2,1);
th20 = data(2,2);
osVg = sum(data(:,3:4),2);
tsVg = sum(data(:,5:6),2);
ndVg = sum(data(:,8:9),2);
splitVg = sum(data(:,11:12),2);

th20_vec = data(:,2);



hfig = figure;
set(hfig,'units','centimeters');
pos = get(hfig,'position');
set(hfig,'position',[pos(1:2),8,6]);
hold on;
h0=plot(th20_vec,osVg,'-.',th20_vec,tsVg,'--',...
                      th20_vec,splitVg,':',th20_vec,ndVg,'-','linewidth',2);
% hleg=legend('OS','TS','SW','ND');
% if th10==th20;
%     set(hleg,'location','northeast');
% else
%     set(hleg,'location','southeast');
% end
% set(hleg,'box','on')

if th10==th20;
    axis([0 3 0 0.15])
    plot(th20_vec,0.05+0*th20_vec,':','linewidth',1);
else
    axis([0 3 0 1]);
    plot(th20_vec,0.05+0*th20_vec,':',th20_vec,0.5+0*th20_vec,'-','linewidth',1);
end
set(gca, 'Box', 'on','xtick',0:.5:3);
if th10==th20;
    ylabel('Rejection Probability','fontsize',11,'fontweight','b','interpreter','none');
    xlabel('$\beta_{0}$','fontsize',11,'fontweight','b','interpreter','none');
    eval(['matlabfrag(''NorLoc_neqs_',num2str(n),'_null.tex'');']);
else
    xlabel('$\beta_{0}$','fontsize',11,'fontweight','b','interpreter','none');
    eval(['matlabfrag(''NorLoc_neqs_',num2str(n),'_power.tex'');']);
end

