%This is the log density function of model \mathcal{F} in the normal
%location example (simulation example 2 of Shi (2011))
%
%The function takes two input variables:
%
%x:     the data vector: d_x\times 1 or d_x\times n
%theta: parameter of model \mathcal{F}
%
%The function produces at most three outputs:
%
%logfi:    \log(f(x,\theta))
%d_logfi:  \partial\log(f(x,\theta))/(\partial\theta)
%d2_logfi: \partial^2\log(f(x,\theta))/(\partial\theta^2): the second
%derivative matrix is vectorized into a row when the x input to the
%function has more than one rows.
%
%In the normal location example, x=(y1,y2). The model assumes that
%(y1,y2)\sim N((theta,0)',I).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [logfi,d_logfi,d2_logfi] = NorLoc_logfi(x,theta)

y1 = x(:,1);
y2 = x(:,2);
nx = length(x(:,1));

logfi = -log(2*pi)/2-(y1-theta).^2/2 - log(2*pi)/2 - y2.^2/2;

if nargout>1;
    d_logfi = (y1-theta);
end

if nargout>2;
    d2_logfi = repmat(-1,nx,1);
end
end

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    



