clear all
macro drop _all
cap log close
log using CoateConlinModels_Estimation, replace text

set more off

use coateconlinaer.dta, clear
global percip rainfall snowfall

***Linear Probability Reduced Form regression***
gen lnf=0
capture program drop lfsur
program lfsur
version 6
args lnf theta1 theta2 sigma1 sigma2 rho
tempvar z
local y1 $ML_y1
local y2 $ML_y2
gen double `z'=(`y1'-`theta1')^2/`sigma1'^2-2*`rho'*(`y1'-`theta1')*(`y2'-`theta2')/`sigma1'/`sigma2'+(`y2'-`theta2')^2/`sigma2'^2
quietly replace `lnf'=-ln(2*_pi*`sigma1'*`sigma2'*sqrt(1-`rho'^2))-`z'/2/(1-`rho'^2) 
quietly replace lnf=`lnf'
end

local forX alc bars citych nonecty weekend $percip avetemp avetemp2 summer perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa evoters
local agaX alc bars citych nonecty weekend $percip avetemp avetemp2 summer perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa evoters
	   
ml model lf lfsur (f: ffor =`forX') (a: fagainst = `agaX') (sigma_f:) (sigma_a:) (rho:),nopreserve
ml maximize

/*Computing the score:*/
ml score fforgrad fagagrad fsigfgrad fsigagrad frhograd
foreach list of local forX {
    gen gradf`list' = `list'*fforgrad
	global gradlist_sur $gradlist_sur gradf`list'
}
global gradlist_sur $gradlist_sur fforgrad
foreach list of local agaX {
    gen grada`list' = `list'*fagagrad
	global gradlist_sur $gradlist_sur grada`list'
	}
global gradlist_sur $gradlist_sur fagagrad fsigfgrad fsigagrad frhograd

matrix sur_Hinv=-e(V)	//Save Heissian 
gen sur_lnf=lnf			//Save log density 
drop lnf

***Group rule-utilitarian model***
gen lnf=0

capture program drop vote
program define vote
version 6

args lnf be xe ce ve we
tempvar v w b c x lnuj bvw eps neps

quietly gen double `v' = exp(`ve')
quietly gen double `w' = exp(`we')
quietly gen double `b' = exp(`be')
quietly gen double `x' = exp(`xe')
quietly gen double `c' = exp(`ce')

quietly gen double `bvw'=exp(lngamma(`v'))*exp(lngamma(`w'))/exp(lngamma(`v'+`w'))

quietly gen double `lnuj' = ln(2)+(`v'-1)*ln($ML_y1)+(`w'-1)*ln($ML_y2)+ ///
(`v'/3)*ln(`v'*`x')+(`w'/3)*ln(`w'*`b')-ln(`bvw')- ///
(`v'+`w')*ln($ML_y1*(`v'*`x')^(1/3)+$ML_y2*(`w'*`b')^(1/3))-.5*ln(2*_pi)

quietly gen double `eps' = ln(`c')+(`v'+`w'+1)*ln((`w'*`b')^(1/3)+(`v'*`x')^(1/3))+ln(`bvw') ///
+2*ln($ML_y1*(`v'*`x')^(1/3)+$ML_y2*(`w'*`b')^(1/3))-ln(`v'+`w')-`v'/3*ln(`v')-`w'/3*ln(`w') ///
-(`v'+3)/3*ln(`x')-(`w'+3)/3*ln(`b')

quietly gen double `neps' =-`eps'^2/2


quietly replace `lnf'= `neps'+`lnuj'
quietly replace lnf=`lnf'

end

constraint define 353 [w]_cons+[b]_cons-[x]_cons-[v]_cons+([b]citych-[x]citych)*0 ///
-[v]perbap*0.5869178 -[v]acctyrlg*0.4480789 -[v]evotov50*0.4772745 -[v]dmsa*0 ///
+([b]alc-[x]alc)*1 +([b]bars-[x]bars)*0 +([b]nonecty-[x]nonecty)*1 -[v]housepri*38058.98  ///
-[v]fsmale*0.0761958 -[v]fhouseo*0.6861603 -[v]fblack*0.0415369 -[v]fmex*0.1211381 =-1.079215

constraint define 361 [w]_cons+[b]_cons-[x]_cons-[v]_cons+([b]citych-[x]citych)*1 -[v]perbap*0.5353338 ///
-[v]acctyrlg*2.542408 -[v]evotov50*0.3917554 -[v]dmsa*0 +([b]alc-[x]alc)*1 +([b]bars-[x]bars)*0 ///
+([b]nonecty-[x]nonecty)*0 -[v]housepri*60790.11 -[v]fsmale*0.0949999 -[v]fhouseo*0.7230927 ///
-[v]fblack*0.1284832 -[v]fmex*0.0873344 =-0.4925168

constraint define 362 [w]_cons+[b]_cons-[x]_cons-[v]_cons+([b]citych-[x]citych)*1 -[v]perbap*0.6068549 ///
-[v]acctyrlg*1.155012 -[v]evotov50*0.52959 -[v]dmsa*0 +([b]alc-[x]alc)*1 +([b]bars-[x]bars)*0 ///
+([b]nonecty-[x]nonecty)*1 -[v]housepri*30597.04 -[v]fsmale*0.0672121 -[v]fhouseo*0.6814431 ///
-[v]fblack*0.0841812 -[v]fmex*0.1184031 =-0.4917728

local grp_bX alc bars citych nonecty
local grp_xX alc bars citych nonecty
local grp_cX weekend $percip avetemp avetemp2 summer
local grp_vX perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa

ml model lf vote (b: ffor fagainst=`grp_bX') ///
(x:`grp_xX')  (c:`grp_cX',noconst) ///
(v:`grp_vX') (w:), constraint(353 361 362)
ml init b:alc=.46 b:bars=-.47 b:citych=1.9 b:nonecty=.51 b:_cons=-2.85 ///
x:alc=.02 x:bars=-1.2 x:citych=1.37 x:nonecty=.7 x:_cons=-1.67 ///
v:perbap=-1.21 v:evotov50=-.08 v:fsmale=1.6 v:fblack=-.05 v:fmex=-1.2 v:fhouseo=.59 v:housepri=0 ///
v:acctyrlg=.03 v:dmsa=-.37 v:_cons=2 w:_cons=1.42 ///
c:weekend=.28 c:rainfall=.002 c:avetemp=.01 c:avetemp2=.000002 c:summer=-1.03

ml maximize

/*Computing the score:*/
ml score grp_bgrad grp_xgrad grp_cgrad grp_vgrad grp_wgrad

foreach list of local grp_bX {
    gen grad_grp_b`list' = `list'*grp_bgrad
	global gradlist_group $gradlist_group grad_grp_b`list'
}
global gradlist_group $gradlist_group grp_bgrad
foreach list of local grp_xX {
    gen grad_grp_x`list' = `list'*grp_xgrad
	global gradlist_group $gradlist_group grad_grp_x`list'
}
global gradlist_group $gradlist_group grp_xgrad
foreach list of local grp_cX {
    gen grad_grp_c`list' = `list'*grp_cgrad
	global gradlist_group $gradlist_group grad_grp_c`list'
}
/*global gradlist_group $gradlist_group grp_cgrad*/
foreach list of local grp_vX {
    gen grad_grp_v`list' = `list'*grp_vgrad
	global gradlist_group $gradlist_group grad_grp_v`list'
}
global gradlist_group $gradlist_group grp_vgrad
global gradlist_group $gradlist_group grp_wgrad

/*Save the Hessian*/
matrix group_Hinv=-e(V)
/*Save the log-density*/
gen group_lnf = lnf
drop lnf

***Intensity model***
gen lnf=0

capture program drop vote
program define vote
args lnf be xe ce ae ve we
tempvar v w b x c a lnuj bvw eps neps
quietly gen double `v' = exp(`ve')
quietly gen double `w' = exp(`we')
quietly gen double `b' = exp(`be')
quietly gen double `x' = exp(`xe')
quietly gen double `c' = exp(`ce')
quietly gen double `a' = exp(`ae')
quietly gen double `bvw'=exp(lngamma(`v'))*exp(lngamma(`w'))/exp(lngamma(`v'+`w'))
quietly gen double `lnuj' = (`v'-1)*ln($ML_y1)+(`w'-1)*ln($ML_y2)+ ///
`v'*ln(`x')+`w'*ln(`b')-ln(`bvw')-(`v'+`w')*ln($ML_y1*`x'+$ML_y2*`b')-.5*ln(2*_pi)
quietly gen double `eps' = ln($ML_y1*`x'+$ML_y2*`b')-ln(`a'*`x'*`b')+ln(`c')
quietly gen double `neps' =-`eps'^2/2
quietly replace `lnf'= `neps'+`lnuj'
quietly replace lnf=`lnf'
end

constraint define 1 [x]_cons=[b]_cons+0.8142215
constraint define 2 [x]_cons=[b]_cons-[x]nonecty+[b]nonecty+1.422203
constraint define 3 [x]_cons=[b]_cons-[x]citych+[b]citych+0.33943
constraint define 4 [x]_cons=[b]_cons-[x]citych+[b]citych-[x]nonecty+[b]nonecty+0.4215914
constraint define 5 [x]_cons=[b]_cons-[x]bars+[b]bars+1.62515
constraint define 6 [x]_cons=[b]_cons-[x]bars+[b]bars-[x]nonecty+[b]nonecty+1.57311
constraint define 7 [x]_cons=[b]_cons-[x]bars+[b]bars-[x]citych+[b]citych+0.4341106
constraint define 8 [x]_cons=[b]_cons-[x]bars+[b]bars-[x]citych+[b]citych-[x]nonecty+[b]nonecty+0.829558
constraint define 9 [x]_cons=[b]_cons-[x]alc+[b]alc+0.6877292
constraint define 10 [x]_cons=[b]_cons-[x]alc+[b]alc-[x]nonecty+[b]nonecty+0.3597384
constraint define 11 [x]_cons=[b]_cons-[x]alc+[b]alc-[x]citych+[b]citych+0.1407262
constraint define 12 [x]_cons=[b]_cons-[x]alc+[b]alc-[x]citych+[b]citych-[x]nonecty+[b]nonecty+0.1639242

local bX alc bars citych nonecty
local xX alc bars citych nonecty
local cX weekend $percip avetemp avetemp2 summer
local aX perbap evotov50 fsmale fblack fmex evoters
local vX perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa

ml model lf vote (b: ffor fagainst=`bX') (x:`xX') ///
(c:`cX',noconst) (a: `aX', noconst) (v:`vX') (w:), constraint(10 12) nopreserve
ml search
ml maximize

/*Computing the score:*/
ml score its_bgrad its_xgrad its_cgrad its_agrad its_vgrad its_wgrad

foreach list of local bX {
    gen grad_its_b`list' = `list'*its_bgrad
	global gradlist_intensity $gradlist_intensity grad_its_b`list'
}
global gradlist_intensity $gradlist_intensity its_bgrad
foreach list of local xX {
    gen grad_its_x`list' = `list'*its_xgrad
	global gradlist_intensity $gradlist_intensity grad_its_x`list'
}
global gradlist_intensity $gradlist_intensity its_xgrad
foreach list of local cX {
    gen grad_its_c`list' = `list'*its_cgrad
	global gradlist_intensity $gradlist_intensity grad_its_c`list'
}
/*global gradlist_intensity $gradlist_intensity grp_cgrad*/
foreach list of local aX {
    gen grad_its_a`list' = `list'*its_agrad
	global gradlist_intensity $gradlist_intensity grad_its_a`list'
}
/*global gradlist_intensity $gradlist_intensity its_agrad*/
foreach list of local vX {
    gen grad_its_v`list' = `list'*its_vgrad
	global gradlist_intensity $gradlist_intensity grad_its_v`list'
}
global gradlist_intensity $gradlist_intensity its_vgrad
global gradlist_intensity $gradlist_intensity its_wgrad

/*Saving the Hessian:*/
matrix intensity_Hinv=-e(V)
/*Saving the log-density:*/
gen intensity_lnf=lnf

log close
